from .reflection import *


class Object(object):
    def __init__(self, fields={}):
        self.__dict__ = fields

    def __getitem__(self, key):
        return self.__dict__[key]

    def __setitem__(self, key, value):
        self.__dict__[key] = value

    def getFieldValue(self, key):
        return self.__dict__[key]

    def setFieldValue(self, key, value):
        self.__dict__[key] = value

    def getFieldKeys(self):
        return getFieldKeys(self)

    def getFieldValues(self):
        return getFieldValues(self)

    def getFields(self):
        return getFields(self)

    def getPublicFields(self):
        return getPublicFields(self)

    def getPrivateFields(self):
        return getPrivateFields(self)

    def getMethods(self):
        return getMethods(self)

    def getPublicMethods(self):
        return getPublicMethods(self)

    def getPrivateMethods(self):
        return getPrivateMethods(self)



from xvg.core import Object
from xvg.math import Vector

import math
import xvg.core
import xvg.math
import xvg.models


class PureContext(Object):

    def __init__(self):
        self.name = 'untitled'
        self.size = Vector()
        self.scale = Vector()

    def setNode(self, id, model):
        pass

    def getNode(self, id):
        pass

    def useNode(self, id):
        pass

    def createGlobalsMap(self, context):
        return {
            "__builtins__": {"locals": locals, "globals": globals}
        }

    def createLocalsMap(self, context):
        localsMap = {}

        localsMap['math'] = math
        localsMap['debug'] = print

        localsMap.update(xvg.core.getModuleClasses(xvg.core))
        localsMap.update(xvg.core.getPublicModuleFunctions(xvg.core))

        localsMap.update(xvg.core.getModuleClasses(xvg.math))
        localsMap.update(xvg.core.getPublicModuleFunctions(xvg.math))

        localsMap.update(xvg.core.getModuleClasses(xvg.models))
        localsMap.update(xvg.core.getPublicModuleFunctions(xvg.models))

        localsMap.update(self.getPublicFields())
        localsMap.update(self.getPublicMethods())

        return localsMap



from .color import *
from .filter import *
from .font import *
from .form import *
from .group import *
from .image import *
from .mask import *
from .path import *
from .point import *
from .text import *



from xvg.core import *


class JSONRenderer():

    def render(self, context):
        result = toJSON(context)
        return FileData(
            value=result,
            type=JSONFileType(),
            name=context.name
        )



class Filter():
    def __init__(self):
        pass


class BatchFilter():
    def __init__(self):
        pass


class BlendFilter():
    def __init__(self):
        pass


class BlurFilter():
    def __init__(self):
        pass


class ShadowFilter():
    def __init__(self):
        pass


class NoiseFilter():
    def __init__(self):
        pass


class ConvolveFilter():
    def __init__(self):
        pass


class DisplaceFilter():
    def __init__(self):
        pass



from .encoding import *
from .error import *
from .filedata import *
from .filepath import *
from .filetype import *
from .json import *
from .list import *
from .object import *
from .query import *
from .reflection import *
from .string import *



class Encoding():
    def __init__(self, *, name):
        self.name = name

    @staticmethod
    def fromName(string):
        lower = string.lower()
        if lower == 'ascii':
            return ASCIIEncoding()
        if lower == 'utf-8':
            return UTF8Encoding()


class ASCIIEncoding(Encoding):
    def __init__(self):
        Encoding.__init__(self, name='ascii')


class UTF8Encoding(Encoding):
    def __init__(self):
        Encoding.__init__(self, name='utf-8')



import xml.etree.ElementTree as ET
from xvg.core import *


class SVGRenderer():
    """
    A Scalable Vector Graphics (SVG) renderer.
        Tutorial: https://developer.mozilla.org/en-US/docs/Web/SVG/Tutorial
        Elements: https://developer.mozilla.org/en-US/docs/Web/SVG/Element
        Attributes: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute
    """

    def __init__(self):
        pass

    def render(self, context):
        """ Renders the image and returns the result """

        headXML = '<?xml version="1.0" standalone="no"?>'
        headDOC = '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">'

        root = ET.Element('svg')
        root.set('xmlns', 'http://www.w3.org/2000/svg')
        root.set('xmlns:xlink', 'http://www.w3.org/1999/xlink')
        root.set('version', '1.1')

        root.set('width', str(context.size.x))
        root.set('height', str(context.size.y))

        viewX = context.scale.x / 2
        viewY = context.scale.y / 2
        viewW = context.scale.x
        viewH = context.scale.y
        root.set('viewBox', f'{viewX} {viewY} {viewW} {viewH}')

        encoding = 'utf-8'  # TODO use class
        result = headXML.encode(encoding)\
            + headDOC.encode(encoding)\
            + ET.tostring(root, encoding)

        return FileData(
            value=result,
            type=SVGFileType(),
            name=context.name
        )



import random


def randomReal(l, r):
    """ Returns a random real number in the range [l, r] """
    return random.uniform(start, stop)


def randomWhole(l, r):
    """ Returns a random whole number in the range [l, r] """
    return random.randrange(start, stop)


def randomGaussian(mu, sigma):
    """ 
    Returns a random in a gaussian distribution.
    mu is the mean and sigma is the standard deviation.
    """
    return random.gauss(mu, sigma)



class ExecInterpreter():

    def __init__(self):
        pass

    def interpret(self, context, script):
        """ Evaluates a script string with a context """
        exec(script,
             context.createGlobalsMap(context),
             context.createLocalsMap(context))

    def interpretFile(self, context, path):
        """ Evaluates a script file with a context """
        with open(path) as file:
            self.run(file.read())



from xvg.contexts import *
from xvg.core import *
from xvg.interpreters import *
from xvg.renderers import *


class Engine():

    def __init__(self, renderer=None, context=None, interpreter=None):
        self.context = context or PureContext()
        self.renderer = renderer or JSONRenderer()
        self.interpreter = interpreter or ExecInterpreter()

    def process(self, script):
        self.interpreter.interpret(self.context, script)
        return self.renderer.render(self.context)

    def processFile(self, pathIn, pathOut=None):
        filePathIn = FilePath(pathIn)
        filePathOut = FilePath(pathOut) if pathOut else FilePath(pathIn)

        script = filePathIn.read()
        result = self.process(script)

        filePathOut.type = result.type
        filePathOut.write(result.value)

    def processFiles(self, pathRoot):
        raise NotImplementedError()



import math
from .vector import *


class Polygon():
    def __init__(self):
        pass


class RegularPolygon():
    def __init__(self, n):
        q1 = math.pi / 2
        q4 = math.pi * 2
        a = q4 / n
        self.vertices = [
            Vector(math.cos(i*a), math.sin(i*a))
            for i in range(n)
        ]



import setuptools
from settings import *

classifierStatus = [
    'Development Status :: 1 - Planning',
    'Development Status :: 2 - Pre-Alpha',
    'Development Status :: 3 - Alpha',
    'Development Status :: 4 - Beta',
    'Development Status :: 5 - Production/Stable',
    'Development Status :: 6 - Mature',
    'Development Status :: 7 - Inactive'
]

classifierPython = {
    '3': 'Programming Language :: Python :: 3 :: Only',
    '2': 'Programming Language :: Python :: 2 :: Only'
}

classifierSystem = {
    'any': 'Operating System :: OS Independent',
    'osx': 'Operating System :: MacOS :: MacOS X',
    'linux': 'Operating System :: POSIX :: Linux',
    'windows': 'Operating System :: Microsoft :: Windows',
}

classifierLicense = {
    'mit': 'License :: OSI Approved :: MIT License',
    'gpl2': 'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
}

# Read the settings
settings = Settings().read()

# Read the README
with open("README.md", "r") as fh:
    readme = fh.read()
    readmeType = "text/markdown"

# Build the package
setuptools.setup(
    name=settings.values.name,
    version=settings.serializeNewVersion(),
    author=settings.values.author,
    description=settings.values.description,
    long_description=readme,
    long_description_content_type=readmeType,
    url=settings.values.repository,
    packages=setuptools.find_packages(),
    classifiers=[
        classifierStatus[settings.values.version.status],
        classifierPython[settings.values.python.split('.')[0].lower()],
        classifierSystem[settings.values.system.lower()],
        classifierLicense[settings.values.license.lower()]
    ],
    python_requires=f'~={settings.values.python}',
)

# Save the settings
settings.write()



import types
import inspect


def getModuleClasses(target):
    return dict(filter(
        lambda kvp: inspect.isclass(kvp[1]),
        inspect.getmembers(target)))


def getModuleFunctions(target):
    return dict(filter(
        lambda kvp: inspect.isfunction(kvp[1]) or inspect.isbuiltin(kvp[1]),
        inspect.getmembers(target)))


def getPublicModuleFunctions(target):
    return dict(filter(
        lambda kvp: kvp[0][0] != '_' and (
            inspect.isfunction(kvp[1]) or inspect.isbuiltin(kvp[1])),
        inspect.getmembers(target)))


def getPrivateModuleFunctions(target):
    return dict(filter(
        lambda kvp: kvp[0][0] == '_' and (
            inspect.isfunction(kvp[1]) or inspect.isbuiltin(kvp[1])),
        inspect.getmembers(target)))


def getFieldKeys(target):
    return vars(target).keys()


def getFieldValues(target):
    return vars(target).values()


def getFields(target):
    return vars(target).items()


def getPublicFields(target):
    return dict(filter(
        lambda kvp: kvp[0][0] != '_',
        vars(target).items()))


def getPrivateFields(target):
    return dict(filter(
        lambda kvp: kvp[0][0] == '_',
        vars(target).items()))


def getMethods(target):
    return dict(filter(
        lambda kvp: isinstance(
            kvp[1], types.FunctionType),
        type(target).__dict__.items()))


def getPublicMethods(target):
    return dict(filter(
        lambda kvp: kvp[0][0] != '_' and isinstance(
            kvp[1], types.FunctionType),
        type(target).__dict__.items()))


def getPrivateMethods(target):
    return dict(filter(
        lambda kvp: kvp[0][0] == '_' and isinstance(
            kvp[1], types.FunctionType),
        type(target).__dict__.items()))



from .encoding import *


class FileType():
    def __init__(self, *, extension, encoding, isBinary):
        self.extension = extension
        self.encoding = encoding
        self.isBinary = isBinary

    @staticmethod
    def fromExtension(string):
        lower = string.lower()
        if lower == '.xvg':
            return XVGFileType()
        if lower == '.json':
            return JSONFileType()
        if lower == '.svg':
            return SVGFileType()
        if lower == '.png':
            return PNGFileType()


class XVGFileType(FileType):
    def __init__(self):
        FileType.__init__(self, extension='.xvg',
                          encoding=UTF8Encoding(), isBinary=False)


class JSONFileType(FileType):
    def __init__(self):
        FileType.__init__(self, extension='.json',
                          encoding=UTF8Encoding(), isBinary=False)


class SVGFileType(FileType):
    def __init__(self):
        FileType.__init__(self, extension='.svg',
                          encoding=UTF8Encoding(), isBinary=True)


class PNGFileType(FileType):
    def __init__(self):
        FileType.__init__(self, extension='.png',
                          encoding=None, isBinary=True)



import os
from .filetype import *


class FilePath():
    def __init__(self, *paths):
        path = FilePath.joinParts(*paths)
        self.root = FilePath.parseRoot(path)
        self.name = FilePath.parseName(path)
        self.type = FilePath.parseType(path)

    def join(self):
        return FilePath.joinParts(
            self.root, self.name + self.type.extension)

    def test(self):
        return os.path.exists(self.join())

    def read(self):
        mode = 'rb' if self.type.isBinary else 'r'
        encoding = None if self.type.isBinary\
            else self.type.encoding.name
        with open(self.join(), mode, encoding=encoding) as file:
            return file.read()

    def write(self, value):
        mode = 'wb' if self.type.isBinary else 'w'
        encoding = None if self.type.isBinary\
            else self.type.encoding.name
        with open(self.join(), mode, encoding=encoding) as file:
            file.write(value)

    @staticmethod
    def joinParts(*parts):
        return os.path.join(*parts)

    def parseRoot(path):
        return os.path.dirname(path)

    @staticmethod
    def parseStem(path):
        return os.path.basename(path)

    @staticmethod
    def parseName(path):
        basename = os.path.basename(path)
        return os.path.splitext(basename)[0]

    @staticmethod
    def parseType(path):
        extension = os.path.splitext(path)[1]
        return FileType.fromExtension(extension)

    @staticmethod
    def search(*, root='./', recursive=False,
               fileFilter=lambda f: True,
               rootFilter=lambda f: True):
        for root, dirs, files in os.walk(root):
            if rootFilter(root):
                for file in files:
                    if fileFilter(file):
                        yield os.path.join(root, file)
            if not recursive:
                break

    @staticmethod
    def match(*, root='./', recursive=False,
              filePattern='.*',
              rootPattern='.*'):
        fileRegex = re.compile(filePattern)
        rootRegex = re.compile(rootPattern)
        return FilePath.search(
            root=root,
            recursive=recursive,
            fileFilter=lambda f: fileRegex.match(f),
            rootFilter=lambda f: rootRegex.match(f)
        )
